<?php

/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the MIT License
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/mit-license.php
 *
 * @category       Shipping
 * @package        Expedipro_Synchro
 * @author         Hilly
 * @copyright      Copyright (c) 2016
 * @license        http://opensource.org/licenses/mit-license.php MIT License
 */
class Expedipro_Synchro_Includes_Controller
{

    private $utils; 	        // use for DI for sub actions classes
    private $format;            //cf. headerEncodingHTTP()



    public function __construct( $actionUtils = null ) {

        if ( ! $actionUtils )
            throw new Exception('We need logger and anwser management');

        $this->utils = $actionUtils;
    }


    /**
      * default action
      *
      * @access public
      * @return void
      */
    public function handleRequest()
    {
        $this->contentNegotiation();
        // tellMe | tellMe & debug | request | request & debug
        $action = $this->extractFromHTTP( );

        // user friendly msg for basic url check
        if ( null === $action ) {
            echo "Hello World";
            return true;
        }

        // Can not use switch since $action can be a scalar false|null or an array with array_keys
        if ( isset( $action['tellMe'] ) )

        {
            if ( ! $this->validateToken( $action['tellMe'] ) )
            {
                $this->utils->answerAddError( 'ACTPRO403', 'Hacking attempt targeting tellMe', $this->httpFeedback() );
                if( isset( $action['debug'] ) )
                    $this->utils->answerAddError( 'ACTPRO000', substr( hash('sha256', $this->getToken() ), 0, 2 ) ); // don't talk too much but help checking with url
                $this->setAnswerData( 401 );
            }
            // tellMe is OK
            $this->sendActionAnswer('tellme','tellMe');
        }

        elseif ( isset( $action['request'] ) )
        {
            if ( ! $container = $this->validateContainer($action['request']))
            {
                $this->utils->answerAddError('ACTPRO405', 'Hacking attempt with wrong container', $this->httpFeedback($container));
                if ( isset($action['debug'])
                    AND $this->validateToken($action['debug'])
                )
                    $this->utils->answerAddError( 'ACTPRO000',
                        $this->validateContainer( $action['request'], true ) // true : debug mode
                    );
                $this->setAnswerData(403);
            }
/*
            $container = array(
                'action'=> 'getOrder',
                'data'  => array( 'orderFrom' => '2012-01-01', 'orderTo' => '2017-01-01' )
            );

            $container = array(
                'action'=> 'setDelivery',
                'data'  => array(
                    'product' =>
                    array(	'R00' => 'Colissimo Access France',
                        'R01' => 'Colissimo Expert France',
                    ),
                    'distribution' =>
                    array(	'AVX' => 'Votre colis est remis',
                            'REN' => 'Livraison effective',
                    ),
                'delivery' =>
                    array(
                        // pending , stand
                        array('order'=> '100000049', 'track'=> 'ABCDEF12', 'step' => '1', 'stepLib' => 'AVX', 'carrier' => '0', 'product' => 'R01', 'insurance' => 234, 'cost' => 15.99),
                        // pending to processing
                 //       array('order'=> '100000049', 'track'=> 'ABCDEF12', 'step' => '10', 'stepLib' => 'AVX', 'carrier' => '0', 'product' => 'R01', 'insurance' => 234, 'cost' => 15.99),
                 //       array('order'=> '100000208-1', 'track'=> 'ABCDEF12', 'step' => '100', 'stepLib' => 'AVX', 'carrier' => '0', 'product' => 'R01', 'insurance' => 234, 'cost' => 15.99),
//                             array('order'=> '100000052-1', 'track'=> 'ZDF543A2', 'step' => '100','stepLib' => 'REN', 'carrier' => '1', 'product' => 'R00','insurance' => 0, 'cost' => 8.22),
                    )
                )
            );

*/
            //
            // from now on, we can safely use $container['action'=>...,'data'=>...,'expire'=>... ]
            //var_dump($this->validateContainer( $_GET['container'] ));
            switch ($container['action'])
            {
                case 'setDelivery' :
                case 'setDeliveryStatus' : // for backward server compliance
                    $container['action'] = 'setDelivery';
                    $this->sendActionAnswer('delivery', $container['action'], $container['data']);
                    break;
                case 'getOrder' :
//                    $this->format['response'] = 'jsonApi'; // force format for legacy browser operations : temporary hack
                    $this->sendActionAnswer('order', $container['action'], $container['data']);
                    break;
                case 'setDeploy' :
                case 'getLog' :
                    $this->sendActionAnswer('maintenance', $container['action'], $container['data']);
                    break;
                default :
                    if (isset($container['data']))  // keep null, could help debug
                        $container['data'] = '<not disclosed>'; // no need to publish data that are not part of the current problem
                    $this->utils->answerAddError('ACTPRO407', 'Unknown action within valid container', $this->httpFeedback($container));
                    $this->setAnswerData(417);
            }
        }
        else
        {
            $this->utils->answerAddError( 'ACTPRO402', 'Hacking attempt : query parameter', $this->httpFeedback() );
            $this->setAnswerData( 400 );
        }
    }


    /**
     * Perform the action and then setAnswerData()
     *
     */
    private function sendActionAnswer($actionType, $action = '', $data = array() ) {
        try {
//$this->utils->logWrite( __FUNCTION__ . json_encode($data) );
            // instantiate the dedicated class and call the abstract like process() method
            if ( ! $instance = $this->utils->getInstance( $actionType ) )
                $this->setAnswerData( 420 );
            else
                $this->setAnswerData( (bool) $instance->process( $action, $data ) ); // error are handled at sub-process level

        } catch ( Exception $e)
        {
            $this->utils->answerAddError( 'ACTPROANS', 'Can not get an answer', $e->getMessage() );
            $this->setAnswerData( 420 );
        }
    }


    /**
     * FINAL step (finish with a die)
     *
     * @param int|bool $httpCode
     *
     */
    private function setAnswerData( $httpCode = 200 ) {

        $this->setAnswerHeader( $httpCode );

        // note that either error or data are returned
        $answer = $this->utils->getAnswer(); // jsonapi
//echo  var_export( $answer );
//$this->utils->logWrite( $this->format );
//$this->utils->logWrite( $answer);
        switch ( $this->format['response'] ) {
            case 'jsonApi':
            case 'json':
                echo rawurlencode ( $this->jsonApiEncode( $answer ) ); //  compliant with RFC 3986
                break;
/*
            case 'text' :
                echo serialize( $answer); //var_export ( $answer, true );
                break;
*/
            case 'csv' : // temp case to preserve browser based get
                // we may have errors
                if ( ! isset( $answer['data']) ) {
                    echo serialize( $answer);
                    break;
                }

                $sep = $this->utils->getConfiguration('orderSeparator');
                // because implode (array(array()) would fail
                $firstLine = true;

                foreach ( $answer as $toplevel => $content)
                { // error or data
                    $answerCsv ='';
                    foreach ( $content as $useless => $data )
                    {
                        /* remove everything that might breaks csv importation
                        - separator
                        - inner line breaks (from textArea)
                        - null
                          alternative : json_decode( str_replace( json_encode() ) )
                        */
                        if ( is_array( $data['attributes'] ) )
                        {
                            if ( $firstLine ) {
                                $answerCsv .= implode( $sep, array_keys( $data['attributes'] ) ). PHP_EOL;
                                $firstLine = false;
                            }
//$this->utils->logWrite( $data['attributes'] );
                            $answerCsv .= implode( $sep, $this->cleanupCsvData( $sep, $data['attributes']) ). PHP_EOL;
//$this->utils->logWrite( $answerCsv );
                        }
                        else
                        {
                            $answerCsv .= $this->cleanupCsvData( $sep, $data['attributes']) . PHP_EOL;
                        }
                    }
                }
                echo $answerCsv;
        }
        die;
    }

    private function cleanupCsvData( $separator = '|', $data = null) {
        if ( null === $data )
            return '';

        if ( is_array( $data ) ) {
            array_walk($data,
                function (&$value, $key, $sep) {
                    // before 5.4, 'this' is not allowed into anonymous function, so we duplicate
                    $value = preg_replace('/\r\n?/', ' ', str_replace($sep, '-', $value) );
                },
                $separator);
            return $data;
        }

        return preg_replace('/\r\n?/', ' ', str_replace($separator, '-', $data) );
    }


    /**
     * Decrypt the container from request and check it comply with expected format
     * Note : the array has been rawurlencode(   base64encode(mcrypt(json_encode()))	)
     *
     * @param string	encrypted array
     *
     * @return false|array
     */
    private function validateContainer ($container = null, $debug = false )
    {
        if ( ! $container )
            return false;

        if ( ! $instance = $this->utils->getInstance('credential') )
            return false;

        if ( $debug )
            return $instance->containerDebug( $container , $this->getToken() );

        if ( ! $assoc = $instance->decrypt( $container , $this->getToken() ) )
            return false;
//var_dump( $assoc );

        // caller decides how long its encrypted request is valid (UTC timestamp)
        if ( ! isset($assoc['data']) OR $assoc['expire'] < time() )
            return false;

        return $assoc;
    }



    /**
     * Whatever the combination of Content-Type and HTTP method, return a normalized associative array
     * Note : ensure that one an only one action (allowed) is provided
     *
     * @param array $parameterAllowed
     *
     * @return array|bool|mixed
     */
    private function extractFromHTTP( $parameterAllowed = array('tellMe', 'request', 'debug' ) ) {

        if ( ! $parameterAllowed )
            return false;
        /*
                                    json                                        urlencoded
                GET         $_GET[ '%7B%22request%22%3A%22..%22%7D'=>'' ]     $_GET[ 'request' =>  ]
                POST        php://input =  '%7B%22request%22%3A%22..%22%7D'   $_POST
        */
        $extract =  array();
        if ( $_SERVER['REQUEST_METHOD'] === 'GET' ) {
            if ( isset($_GET) AND is_array($_GET) ) {
//$this->utils->logWrite($_GET );
/*
                if ( $this->format['request'] === 'json' OR $this->format['request'] === 'jsonApi')
                {
                    // $_GET = array( '{"tellMe":"8d4831220851452793f3a2a01f4b1bf4f8e0a924d22a7ff964c13d921b8fa891"}' => '' )


                    // true to force assoc array (instead of object)
                    // some server escape $_GET string with \ and make it undecodable
                    $extract = rawurldecode( array_keys ($_GET) ) // json_decode( rawurldecode( array_keys ($_GET) ), true )
                                ?: rawurldecode( $_SERVER['QUERY_STRING']);
                }
                else
*/
                    $extract = $_GET; // PHP manage decode and array alone
            }
        } else
        {   // POST, PATCH, DELETE
//var_export( $_POST );
//$this->utils->logWrite($_POST );
            if ( isset($_POST) AND is_array($_POST) AND $_POST ) {
                $extract = $_POST;

            } else {
                $arrayIndex = file('php://input'); // < 5.2.1 quite crap, < 5.6 read only once
                foreach( $arrayIndex as $index => $urlParam ) {
                    $tmp = explode( '=', $urlParam ) ?: array( 0=>'fake', 1=>'' );
                    $extract[ $tmp[ 0 ] ] = $tmp[ 1 ];
                }
//$this->utils->logWrite($body );
//                if ( strlen( trim($body) ) > 0)
//                    $extract = rawurldecode( $body);
            }
        }
//$this->utils->logWrite(var_export( $extract, true ) );

        if (!  $extract )
            return null;

        // only one parameter is allowed
        $action= array_keys($extract); // array of keys
        if ( count( $action ) > 1 and ! in_array( 'debug', $action ) )
            return false;

//var_export ( array_intersect_key( $extract, array_fill_keys($parameterAllowed , true ) ) );

        // the action must be defined as allowed
        $extract = array_intersect_key( $extract, array_fill_keys($parameterAllowed , true ) );
        // the action must have a value
        if ( ! $extract )
            return false;

        foreach( $action as $toDecode )
            $extract[ $toDecode ] = rawurldecode( $extract[ $toDecode ] );
//$this->utils->logWrite(var_export( $extract, true ) );

        return $extract;
    }


    /**
     * Manage HTTP headers for both request and response
     * http://jsonapi.org/format/#content-negotiation
     *
     * Note update global $this->format
     */
    private function contentNegotiation()
    {

        $this->format = array("request" => 'urlencoded', "response" => 'csv');    // default

//      $contentTypeJson = array( 'application/json', 'application/vnd.api+json');
        $type = isset($_SERVER['CONTENT_TYPE'])
            ? $_SERVER['CONTENT_TYPE']
            : (isset($_SERVER['HTTP_CONTENT_TYPE']) ? $_SERVER['HTTP_CONTENT_TYPE'] : '');
        // content-type may include many types by priority order (not taken into account here)
        if (stristr($type, 'application/json')) {
            $this->format['request'] = $this->format['response'] = 'json';
        }
        elseif (stristr($type, 'application/vnd.api+json')) {
            $this->format['request'] = $this->format['response'] = 'jsonApi';
        }

        // 2016 september : we are so far from doing a reel web service jsonapi compliant...
        // forget about input format, it will always be like ?request=urlencoded data
        // however, due to legacy compliance, expedipro.com may expect answer in two different formats
        $this->format['request'] = 'urlencoded';

        // http://jsonapi.org/format/#content-negotiation forbid usage of HTTP_ACCEPT
/*
        // handle the encoding of data to send back (response)
        $type = isset($_SERVER['HTTP_ACCEPT']) ? $_SERVER['HTTP_ACCEPT'] : '';
        if (stristr($type, 'application/vnd.api+json'))
            $this->format['response'] = 'jsonApi';
        elseif (stristr($type, 'json'))
            $this->format['response'] = 'json';
*/
//$this->utils->logWrite( $this->format);
    }


    private function setAnswerHeader( $httpCode = 200 ) {
        $httpStatus = array(
            // jsonApi status
//            'fetched'       => [200, '200 OK'],
//            'created'       => [201, '201 Created'],
//            'patched'       => [200, '200 OK'],
//            'deleted'       => [200, '200 OK'],
            '200'   =>  ' OK',
            '400'   =>  ' 400 Bad request',
            '401'   =>  ' 401 Unauthorized',
            '403'   =>  ' 403 Forbidden',
            '412'   =>  ' 412 Precondition Failed',
            '417'   =>  ' 417 Expectation Failed',
            '420'   =>  ' 420 Method Failure'
        );

        switch ($this->format['response'] ) {
            case 'json' :
                header('Content-Type: application/json'); // ; charset=utf-8');
                break;
            case 'jsonApi' :
                header('Content-Type: application/vnd.api+json'); // ; charset=utf-8');
                // do not set HTTP_ACCEPT : http://jsonapi.org/format/#content-negotiation
                break;
//            case 'csv':
//            case 'text':
            default :
                header('Content-Type: text/plain;  charset=UTF-8'); // ; charset=utf-8');
        }

        if ( $httpCode === true )
            $httpCode = 200;
        elseif ( $httpCode === false )
            $httpCode = 420;
        elseif ( ! isset($http[$httpCode]) )
            $httpCode = 420;

        if ( intval($httpCode) >= 400)
            header($_SERVER["SERVER_PROTOCOL"].$httpStatus[$httpCode], true, $httpCode);


        // https://developer.mozilla.org/en-US/docs/Web/HTTP/Server-Side_Access_Control
        header('Access-Control-Allow-Origin: *');
    }


    /**
     * Produce a very simple primary data resource or resources collections
     * http://jsonapi.org/format/#document-top-level
     *
     * @param array $data
     *
     * @return string
     */
    private function jsonApiEncode( $answer = array() ) {
        // avoid sending a collection of single resource
//$this->utils->logWrite( $answer);

        if ( isset( $answer['data']) AND count($answer['data']) === 1 )
            $answer['data'] = array_pop( $answer['data'] );
        if ( isset( $answer['error']) AND count($answer['error']) === 1 )
            $answer['error'] = array_pop( $answer['error'] );
//$this->utils->logWrite( $answer);

        return json_encode( $answer );
    }


    /**
     * Provide context elements for error at request level
     *
     * @param void $misc
     *
     * @return associative array
     */
    private function httpFeedback ($misc = null ) {
        $addr = '';
        // HTTP_X_FORWARDED_FOR can easily be forged but useful for NAT address
        // REMOTE_ADDR can hardly be forged
        if (isset ( $_SERVER ['HTTP_X_FORWARDED_FOR'] ))
            // http://en.wikipedia.org/wiki/X-Forwarded-For
            $addr .= str_replace ( ',', '_', str_replace ( ' ', '', $_SERVER ['HTTP_X_FORWARDED_FOR'] ) ) . '_';

        $feedback = array(
            "REMOTE_ADDR"       => $addr . $_SERVER ['REMOTE_ADDR'],
            "SERVER_PROTOCOL"   => $_SERVER["SERVER_PROTOCOL"],
            "GATEWAY_INTERFACE" => $_SERVER["GATEWAY_INTERFACE"],
            "REQUEST_METHOD"    => $_SERVER["REQUEST_METHOD"] );

        if ( isset( $_SERVER["REDIRECT_URL"]) ) {
            $feedback["REDIRECT_URL"]           = $_SERVER["REDIRECT_URL"];
            $feedback["REDIRECT_QUERY_STRING"]  = $_SERVER["REDIRECT_QUERY_STRING"];
        }

        if ( isset( $_SERVER["REQUEST_URI"]) ) {
            $feedback["REQUEST_URI"]    = $_SERVER["REQUEST_URI"];
            $feedback["QUERY_STRING"]   = $_SERVER["QUERY_STRING"];
        }

        $feedback['requestType'] = $this->format['request'];
        $feedback['responseType'] = $this->format['response'];

        if ( $_POST )
            $feedback["POST"] = $_POST;

        if ( $misc )
            $feedback['misc'] =$misc;

        return $feedback;
    }

    
    private function validateToken( $tellMe = null ) {
        if ( ! $token = $this->getToken() )
            return false;
        return ( $tellMe === hash('sha256', $token ) );
    }



    protected function getToken( ) {
        if ( ! $token = $this->utils->getConfiguration('token', 'expedipro') )
        {
            $this->utils->answerAddError( 'ACTPRO401', 'Can not get token', $this->httpFeedback() );
            $this->setAnswerData( 420 );
            return false;
        }
//echo hash('sha256', $token );
        return $token;
    }


}
