<?php

/**
 * Class Expedipro_Synchro_Includes_Credential
 *
 * This source file is subject to the MIT License
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/mit-license.php
 *
 * @category       Shipping
 * @package        Expedipro_Synchro
 * @author         Hilly
 * @copyright      Copyright (c) 2016
 * @license        http://opensource.org/licenses/mit-license.php MIT License
 */
class Expedipro_Synchro_Includes_Credential
{
    private $utils;

    // DI of utils
    public function __construct( $utils = null )
    {
        $this->utils = $utils;
    }


    /**
     * Generate a random string
     *
     * @param number $length (default is 32 like ebay)
     *
     * @return string
     */
    public function expedipro_randomString($length = 32)
    {
        // must be compliant with url encoding (not sure about Tools::getValue() regexp)
        $charSetKey = str_shuffle("0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ");
        $charSetSize = strlen($charSetKey); // mb_strlen( $charSetKey, 'UTF-8' ) -1;

        $result = '';
        for ($p = 0; $p < $length; $p++) {
            /* safe only when $charSetKey contains single Byte chars */
            $result .= $charSetKey[mt_rand(0, $charSetSize - 1)];
        }
        return str_shuffle($result);
    }


    /**
     * Decrypt a string encrypted with MCRYPT_RIJNDAEL_128
     *
     * @param string $encrypted that must be decrypted
     * @param string $key       encryption key
     *
     * @return false|string
     */
    public function decrypt($encrypted, $key = '')
    {
        if ( ! $key )
            return false;
        /**
         * not sure mcrypt_get_iv_size and mcrypt_decrypt is allowed by all providers
         */
        $encrypted = rawurldecode($encrypted);

        $key = hash('SHA256', $key . $key, true);    //done also on expedipro size, produce 32 binary
        $iv = base64_decode( substr($encrypted, 0, 22) . '==');
        if (strlen($iv) !== 16)
            $iv = '\0\0\0\0\0\0\0==';

        $encrypted = substr($encrypted, 22);
        $decrypted = rtrim(mcrypt_decrypt(MCRYPT_RIJNDAEL_128, $key, base64_decode($encrypted), MCRYPT_MODE_CBC, $iv), "\0\4");
        $hash = substr($decrypted, -32);
        $decrypted = substr($decrypted, 0, -32);
        // Integrity check. If this fails, either the data is corrupted, or the password/salt was incorrect.


        // prior PHP 5.3 rawurlencode was based on RFC1738 a-while after (expedipro side) it use RFC3986
        if (version_compare(PHP_VERSION, '5.3.4', '>') AND md5($decrypted) !== $hash)  // bypass md5 check prior to 5.3.4, explore string json
            return false;

        // basic checks of right decrypt and consistency : means caller knows token + method + structure
        if (false === stristr($decrypted, '{"action":') OR false === stristr($decrypted, '"expire":'))
            return false;

        return json_decode($decrypted, true);    // associative array
    }


    public function containerDebug($encrypted = null, $key = '')
    {
        if (!$encrypted) {
            echo "empty container";
            return false;
        }

        if ( ! $key ) {
            echo "empty key";
            return false;
        }

        $output = '';

        /*
          SDZy8%2BLrWCH7Y4q%2FvZ0%2BpgXGySD3sgj7iwP0TKF2T3YZaZdTQbZ1fZG%2B6Sytzg19ey0KNpOGvJQ%2BYAMKMasvhCjnGJTFchB4IP5Jhi0qnNrU0D87gqWekA3Zuje5XEUZKPbHhobzWyGTG%2BNoW5V%2FU5xtq%2BX4RQGGB0trLlyuF8sjrv3ECYqqd4FM8ZnrfTg9w%3D"
        */

        $encrypted = rawurldecode($encrypted);
        $key = hash('SHA256', $key . $key, true);
        $iv = base64_decode(substr($encrypted, 0, 22) . '==');

        if (strlen($iv) !== 16) {
            $iv = '\0\0\0\0\0\0\0==';
            $output .= print_r("iv, \0 filled:".$iv, true);
        } else
            $output .= print_r("iv:".$iv, true );

                $encrypted = substr($encrypted, 22);
        $decrypted = rtrim(mcrypt_decrypt(MCRYPT_RIJNDAEL_128, $key, base64_decode($encrypted), MCRYPT_MODE_CBC, $iv), "\0\4");

        $output .= var_export("decrypted:". $decrypted, true);
        $output .= var_export("md5 expedipro:". substr($decrypted, -32), true );
        $output .= var_export("md5 module:". md5($decrypted), true );
        return $output;
    }
}
?>