<?php
/**
 * Class Expedipro_Synchro_Includes_Maintenance
 *
 * This source file is subject to the MIT License
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/mit-license.php
 *
 * @category       Shipping
 * @package        Expedipro_Synchro
 * @author         Hilly
 * @copyright      Copyright (c) 2016
 * @license        http://opensource.org/licenses/mit-license.php MIT License
 */
class Expedipro_Synchro_Includes_Maintenance
{
/*
 //  TODO not yet migrated from Prestashop

    private     $backup;        // set in backup()
    private     $newVersion;    // set in getNewVersion()
    private     $fileTargeted;  // IMPORTANT : use the same target for cleanup and backup
    private     $currentDir; // avoid calling n times $this->utils->getCurrentDir()
*/

    private     $utils;


    public function __construct( $actionUtils = null ) {

        if ( ! $actionUtils )
            throw new Exception('We need logger and answer management');

        $this->utils = $actionUtils;
        $this->currentDir = $this->utils->getCurrentDir(); // .'TEST';
    }


    /**
     * Handle the get or post request before main action
     *
     * @return array error|data
     */
    public function process( $action, $data )
    {
//var_export( $data );
        switch ( $action )
        {
            case 'getLog' :
//                $this->utils->logWrite( __FUNCTION__ .json_encode($data) );
                $answer = $this->getLog( ( isset( $data['reset']) ? $data['reset'] : false ) );
                $this->utils->logWrite( '##### log has been extracted ') ;
                return $answer;

            case 'setDeploy' :
/*
    //        TODO not yet migrated from Prestashop

                if (version_compare(phpversion(), '5.3.1', '<')) {
                    $this->utils->answerAddError('ACTMAI410', 'PHP 5.3+ is required', 'current is '.phpversion()) ;
                    return false;
                }

                if ( ! ( isset( $data['version']) AND
                    isset( $data['source']) AND
                    isset( $data['url']) AND
                    isset( $data['checksum']) AND
                    (isset($data['method']) AND ( $data['method'] === 'patch' OR $data['method'] === 'clean' ) )
                ) )
                {
                    $this->utils->answerAddError('ACTMAI411', 'Missing required data fields', __CLASS__.'->'.__FUNCTION__ ) ;
                    return false;
                }

                return $this->setDeploy($data['method'],
                    $data['source'],
                    $data['version'],
                    $data['url'],
                    $data['checksum'],
                    ( isset( $data['riskFriendly'] ) ? $data['riskFriendly'] : false ) );
*/
        }
        return false;
    }


    /**
     * Return the log
     *
     * @param bool $truncate to truncate log file
     *
     * @return bool
     */
    private function getLog( $truncate = false )
    {
//        $this->utils->logWrite( __FUNCTION__ . ($truncate ? 'oui' : 'non') );
        $status = $this->utils->logRead( $truncate );// provide also a mean to reduce log size
        if ( $status === false )
            return false;

        $this->utils->answerAddData( 'extract', 'log', $status );
        return true;
    }



    private function setDeploy($method, $source, $version, $link, $checksum, $riskFriendly = false )
    {

        if ( ! Configuration::get('EXPEDIPRO_ALLOWUPDATE') ) {
            $this->utils->answerAddError('ACTMAI412', 'Update is not allowed');
            return false;
        }

        $this->fileTargeted = '/\.php|\.tpl|\.png|\.gif$/i';
        $this->utils->makeDir( $this->currentDir .'/deploy' );
        $this->utils->makeDir( $this->currentDir .'/upgrade' );

        if ( ! $this->checkNewVersion( $source, $checksum, $version ) )
            return false;

        if ( ! $this->getNewVersion( $link, $checksum, $version ) )
            return false;
//print_r( $this->newVersion ); echo PHP_EOL;

        if ( ! $this->backup( $version, $riskFriendly ) )
            return false;
//print_r( $this->backup ); echo PHP_EOL;

        if ( ! $this->extract( $method ) ) {
            $this->restore();
            return false;
        }

        if ( ! $this->afterward( $version ) ) {
            $this->restore(); // make sure that any change (e.g. db, hook, prestashop config...) have been rollback first within afterward
            return false;
        }

        $this->utils->answerAddData( 'ACTMAI203', 'deploy', 'deployment done without error' );
        $this->utils->logWrite( $this->utils->getAnswer() ); // usually, positive answer are not logged, but here it worse it

        return true;
    }



    /**
     * Create a tar file under <version> directory from $link, check also its checksum
     * Note : set $this->newVersion
     *
     * @param string    $link to the uri
     * @param string    sha1 checksum : We could rely on tar self signature (for integrity) but this checksum also protect from archive substitution
     * @param string    version like "2.6.0", "last" by default
     *
     * @return boolean
     */
    private function checkNewVersion( $source, $checksum, $version = 'last' )
    {
        $link = 'http://www.expedipro.com/module/checksum?module=OPR&source='.$source.'&version='.$version.'&sha1='.$checksum;
        try {
            $options = array(
                CURLOPT_URL 			=> $link,
                CURLOPT_CONNECTTIMEOUT	=> 3*60, // before getting server entry connection
                CURLOPT_TIMEOUT			=> 5*60,// before getting output server answer
                CURLOPT_FOLLOWLOCATION  => true,
                CURLOPT_FAILONERROR		=> true, // any HTTP response code greater than 400 cause an error, instead of returning the page HTML
                CURLOPT_USERAGENT		=> 'Mozilla/5.0 (Windows NT 5.1; rv:31.0) Gecko/20100101 Firefox/31.0', // not empty to avoid rejection from Apache rules
                CURLOPT_HTTPHEADER      =>
                    array(  'Content-type: application/x-www-form-urlencoded',
                            'Accept: application/vnd.api+json' )
            );

            $curl = curl_init();
            curl_setopt_array($curl, $options );
            $result = curl_exec($curl);
            /*
            var_export($link);
            echo "\n";
            var_export($result);
            echo "\n";

                        $error = ( curl_getinfo($curl, CURLINFO_HTTP_CODE) >= 400 );// HTTP_CODE returned as int
            var_export($error);
            echo "\n";
            */
            if ( ! $result )
                throw new Exception( 'errno: '.curl_errno($curl) . ', error: '.curl_error($curl) . ', getinfo: '. print_r(curl_getinfo($curl), true) );

            curl_close($curl);
            return true;

        } catch (Exception $e) {
            $this ->utils->answerAddError( 'ACTMAI413', 'Fail while checking the source of this new version', $e->getMessage() );
            return false;
        }
    }


    /**
     * Create a tar file under <version> directory from $link, check also its checksum
     * Note : set $this->newVersion
     *
     * @param string    $link to the uri
     * @param string    sha1 checksum : We could rely on tar self signature (for integrity) but this checksum also protect from archive substitution
     * @param string    version like "2.6.0", "last" by default
     *
     * @return boolean
     */
    private function getNewVersion( $link, $checksum, $version = 'last' )
    {
        try {
            $module = $this->currentDir .'/deploy/'. $version ;
            $this->utils->makeDir( $module );

            $module = $module .'/module.tar';
//print_r( $module );
//echo PHP_EOL;
            $archive = fopen($module, "w"); // overwrite

            $options = array(
                CURLOPT_URL 			=> $link,
                CURLOPT_CONNECTTIMEOUT	=> 3*60, // before getting server entry connection
                CURLOPT_TIMEOUT			=> 5*60,// before getting output server answer
                CURLOPT_FOLLOWLOCATION  => true,
                CURLOPT_FILE	        => $archive,
                CURLOPT_FAILONERROR		=> true, // any HTTP response code greater than 400 cause an error, instead of returning the page HTML
                CURLOPT_USERAGENT		=> 'Mozilla/5.0 (Windows NT 5.1; rv:31.0) Gecko/20100101 Firefox/31.0' // not empty to avoid rejection from Apache rules
            );

            $curl = curl_init();
            curl_setopt_array($curl, $options );
            $result = curl_exec($curl);

            // even with crap url, dropbox send http_code 200
//            $error = ( curl_getinfo($curl, CURLINFO_HTTP_CODE) >= 400 );// HTTP_CODE returned as int
//            if ( $error )
//                throw new Exception( 'errno: '.curl_errno($curl) . ', error: '.curl_error($curl) . ', getinfo: '. print_r(curl_getinfo($curl), true) );

            fclose( $archive );
            curl_close($curl);
            /*
            echo PHP_EOL;
            print_r( $checksum );
            echo PHP_EOL;
            print_r( sha1_file( $module ) );
            echo PHP_EOL;
            */
            if ( $checksum !== sha1_file( $module ) )
                throw new Exception( 'invalid checksum or file not found'.$link .' :: ' . $checksum );

            $this->newVersion = $module;
            return true;

        } catch (Exception $e) {
            $this ->utils->answerAddError( 'ACTMAI414', 'Can not get and store module source', $e->getMessage() );
            return false;
        }
    }


    /**
     * Operations done BEFORE the actual deployment (at least a backup)
     * Note : set $this->backup
     *
     * @param   string  version like "2.6.0"
     * @param   bool    $riskFriendly, if true backup becomes best effort based
     *
     * @return boolean
     */
    private function backup( $version, $riskFriendly = false )
    {
        try {

            // this directory is going to be used also for storing .tar, so it is a must have
            $dir = $this->currentDir . '/deploy/' . $version;
            if ( ! is_dir ($dir ) AND ! mkdir( $dir ) ) // avoid fopen warning
                throw new Exception('Can not create directory '. $dir );

            $file = $dir .'/backup.tar';
            // http://php.net/manual/fr/phar.fileformat.comparison.php
            $phar = new PharData( $file );
            if ( ! $phar->isWritable() ) {  //TRUE if phar.readonly is 0, and the actual phar archive is not read-only
                unset($phar);
                if ( ! $riskFriendly )
                    throw new Exception('Phar is readonly in php.ini');

                return true;
            }

            // add all files in the project
            $saved = $phar->buildFromDirectory( $this->currentDir, $this->fileTargeted ); // avoid saving previous tar, *deploy.php is not targeted
//var_export ( $debug );echo PHP_EOL;
            unset($phar);

            if ( count($saved) < 2 )
                throw new Exception('Backup failed (looks empty)');

            $this->backup = $file;
            return true;

        } catch (Exception $e) {
            $this ->utils->answerAddError( 'ACTMAI416', 'Can not backup current module', $e->getMessage() );
            return false;
        }
    }



    /**
     * Extract newVersion file
     * Important : Include here operations that can be rollback with restore())
     *
     * @param string    method
     * 'patch' : just extract archive files without concern about other files
     * 'clean' : remove previous files
     *
     * @return boolean
     */
    private function extract( $method )
    {
        try {
            // step #1 : open the archive and check its content (no extract yet)
            if ( ! ($this->currentDir AND $this->newVersion ) ) // check currentDir for later delTree
                throw new Exception("New module version can not be found");

            $phar = new PharData($this->newVersion);
            if ( ! $phar->isFileFormat( Phar::TAR ) )
                throw new Exception("File format is not tar");

            $found = 0;
            foreach (new RecursiveIteratorIterator($phar) as $file) {
                $found = $found + (int) preg_match($this->fileTargeted, $file);
            }

            if ( ! $found )
                throw new Exception('No targeted file found in new version archive '); // should not happen

            if ( $method === 'clean' AND ! isset( $phar['expedipro']) )
                throw new Exception('Clean update must comply with standard install');
            elseif ( $method === 'patch' AND isset( $phar['expedipro']) )  // patch is supposed to have very few files, this is not a full install
                throw new Exception('Patch update is not compatible with standard install ');


            // step #2 : clean up if necessary
            if ( file_exists( $this->currentDir.'/expedipro' ))
                $this->delTree( $this->currentDir.'/expedipro' ); // in case it remains from previous install

            if  ( $method === 'clean'
                AND ( $fileBefore = $this->foundFiles( $this->currentDir, $this->fileTargeted ) ) ) // same regexp than backup
                array_map('unlink', $fileBefore );


            // step #3 : Extract the archive as it
            // top level is /expedipro directory that can not be bypassed (looks buggy in PHP 5.3 and 5.4)
            $phar->extractTo( $this->currentDir, null, true); // extract all files and overwrite
            unset($phar);


            // step #4 : move new files if necessary
            if ( $method === 'patch' )
                return true;

            $this->utils->copy_r( $this->currentDir.'/expedipro', $this->currentDir );
            $this->utils->delTree( $this->currentDir.'/expedipro' );
            return true;

//    echo "\n"."\n"."\n".$phpfile."\n"."\n"."\n";

        } catch (Exception $e) {
            $this ->utils->answerAddError( 'ACTMAI417', 'Can not extract new module', $e->getMessage() );
            return false;
        }
    }


    /**
     * Execute post deployment instructions, if any
     * Note : try to comply to Prestashop auto upgrade rules
     * Instruction : the user_func should handle itself the rollback since we
     *
     * @param string    version like "2.6.0"
     *
     * @return boolean
     */
    private function afterward( $version )
    {
        try {
            $upgrade = $this->currentDir .'/upgrade/install-'.$version.'.php';
            // Check if folder exist and it could be read
            if ( ! file_exists($upgrade) )
                return true; // regular use case : just replacing file, not post operations
            /*
            echo PHP_EOL;
            echo "install file " . $upgrade;
            echo PHP_EOL;
            */
            include( $upgrade );

            // having such file means something MUST be done
            $version = str_replace('.', '_', $version); // comply to http://doc.prestashop.com/display/PS15/Auto-updating+modules

            if ( ! is_callable( 'upgrade_module_'.$version ) )
                throw new Exception('upgrade_module_'.$version.'() can not be found in '.$upgrade );

            if ( call_user_func('upgrade_module_'.$version ) )
                return true;    // regular case : everything is fine


            if ( ! is_callable( 'rollback_module_'.$version ) )
                throw new Exception('upgrade failed but rollback_module_'.$version.'() can not be found in '.$upgrade );

            if ( ! call_user_func('rollback_module_'.$version ) )
                throw new Exception('can not rollback with success');

            throw new Exception('upgrade failed with successful rollback');


        } catch (Exception $e) {
            $this ->utils->answerAddError( 'ACTMAI418', 'Can not execute steps afterward', $e->getMessage() );
            return false;
        }
    }


    /**
     * Rollback deployment using the backup file
     *
     * @return bool
     */
    private function restore( )
    {
        try {

            if ( ! $this->backup )
                return true;

            $phar = new PharData($this->backup);
            if ( $phar->extractTo($this->currentDir, null, true) ) // extract all files and overwrite // dirname(__FILE__)
                throw new Exception('Backup extraction return false (but without exception) ');

            unset($phar);
            return true;

        } catch (Exception $e) {
            $this ->utils->answerAddError( 'ACTMAI419', 'Can not restore backup', $e->getMessage() );
            return false;
        }
    }


    private function foundFiles( $dir, $pattern )
    {
        $directory = new RecursiveDirectoryIterator( $dir );
        //we need to iterate recursively
        $it = new RecursiveIteratorIterator($directory);
        //Now, match files
        $found = new RegexIterator($it, $pattern );
        unset( $directory, $it);
        if ( ! $found = iterator_to_array($found) )
            return false;
/*
// for debug purpose
foreach($found as $v){
    echo $v."\n";
}
*/
        return $found;
    }

}
?>