<?php
/**
 * Class Expedipro_Synchro_Includes_Order
 *
 * This source file is subject to the MIT License
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/mit-license.php
 *
 * @category       Shipping
 * @package        Expedipro_Synchro
 * @author         Hilly
 * @copyright      Copyright (c) 2016
 * @license        http://opensource.org/licenses/mit-license.php MIT License
 */
class Expedipro_Synchro_Includes_Order
{
    private $utils;
    private $line;
    private $gender = array( '1' =>'H', '2'=>'F');
    private $separatorInner = 'ǁ';
    private $template = array(
        'client'		=> '',
        'civilite'		=> '',
        'prenom' 		=> '',
        'nom' 			=> '',
        'entreprise' 	=> '',
        'entrepriseNom'	=> '',
		'entrepriseTva'	=> '',
    	'optin'			=> '',
        'adresse1'		=> '',
        'adresse2'		=> '',
        'adresse3'		=> '',
        'cp'			=> '',
        'ville'			=> '',
        'pays'			=> '',
        'instruction'	=> '',
		'telephoneF'	=> '',
        'telephoneM'	=> '',
        'email'			=> '',
        'canal'	    	=> '',
        'reference'		=> '',
        'date'		    => '',
        'iteration'	    => '',
        'pDesc'	        => '',
        'pQte'		    => '',
        'pPoids'	    => '',
        'pVal'	        => '',
        'pCateg'        => '',
        'pPays'	        => '',
        'long'		    => '',
        'larg'		    => '',
        'haut'		    => '',
        'livReseau'	    => '',
        'livOption'     => '',
        'livPoint'      => '',
        'allotir'		=> '',
    );


    public function __construct( $actionUtils = null ) {

        if ( ! $actionUtils )
            throw new Exception('We need logger and anwser management');

        $this->utils = $actionUtils;
    }


    /**
     * Generic
     * @param $action
     * @param $data
     *
     * @return bool
     */
	public function process( $action, $data )
	{
        switch ( $action ) {
            case 'getOrder' :
//                $from = ( Validate::isDate($from) ? $from : null);
//                $to = ( Validate::isDate($to) ? $to : null);
                return $this->getOrder(
                                    ( isset( $data['orderFrom'])? $data['orderFrom']: "2016-01-01" ),
                                    ( isset( $data['orderTo'] )	? $data['orderTo']	: 'now()' ),
                                    ( isset( $data['debug'])    ? (bool) $data['debug'] : false ) );
        }
        return false;
	}


    /**
     * Extract orders from model and export them
     *
     * @param string	$from mysql date	inclusive start time interval
     * @param string	$to					inclusive end time interval
     * @param boolean	$debug				true just ignore previous parameters (debug purpose)
     *
     * @return false | object resource collection
     */
    protected function getOrder($from = '', $to = '', $debug = false )
    {
        try {
            // Should we manage sql limit ?

            // https://github.com/OpenMage/magento-mirror/blob/magento-1.7/lib/Varien/Data/Collection/Db.php


            // sales/order.updated_at date can change if you add a comment to the order or some cron/payment gateway do something
            $collection = Mage::getModel('sales/order')
                ->getCollection();

            // order_status_history only record PAST events; so order in status 'processing' has a single line of history with status 'pending'
/*
            $collection->getSelect()
                ->join( array('osh' => $collection->getTable('sales/order_status_history')),
                        'osh.parent_id = `main_table`.entity_id AND osh.entity_name = "order"',
                        array('created_at') )
                ->group('main_table.entity_id');
*/
            // do that after getSelect to addAttributes to osh.*
            $collection
//                ->addAttributeToFilter( 'osh.created_at', array('from' => $from, 'to' => $to) )
//                ->addAttributeToFilter( 'osh.status', array('in' => array( $this->utils->getConfiguration('orderStatus') )))
//                ->addAttributeToFilter( 'store_id',  array('eq' => $storeId ) )
                ->addAttributeToFilter( 'main_table.updated_at', array('from' => $from ) )  /* NH 2017-11 remove this because we receive a now() that becomes 'now()' , 'to' => $to) ) */
                ->addAttributeToFilter( 'main_table.status', array('in' => array( $this->utils->getConfiguration('orderStatus') )))
                ->addAttributeToFilter( 'main_table.shipping_method', array('in' => array( $this->utils->getConfiguration('orderCarrier') )))
            ;
//var_dump( (string) $collection->getSelect() );
        } catch (Exception $e) {
//var_dump( $e );
            $this->utils->answerAddError( 'ACTORD401', 'Can not access DB tables: ' . $e->getMessage() );
            return false;
        }

        try {
            return $this->exportOrder( $collection );
        } catch (Exception $e) {
//       var_dump( $e );
            $this->utils->answerAddError( 'ACTORD402', 'Can export orders find in DB: ' . $e->getMessage() );
            return false;
        }

    }


	/**
	 * get orders related data and save them to answer
     *
     * @param object resource collection
	 *
	 * @return boolean
	 */
    protected function exportOrder( $orderCollection )
	{
$this->utils->logWrite( $orderCollection->toArray() );
//        $this->addToFirstLine();

		if ( ! $orderCollection )
            return true;

        $carriers = $this->utils->getMapping( 'carrier');
        // we could assign current store to all orders 
//        $canal = Mage::app()->getStore( )->getName();
//$this->utils->logWrite($orderCollection );
		foreach($orderCollection AS $order)
		{
//print_r($order->toArray() );
            $this->line = $this->template;

            $this->addToLine( array('reference' => $order->getIncrementId() ) );
            $this->addToLine( array('iteration' => $order->getId() ) );
            /*
                Mage::app()->getStore( $order->getData('store_id') )->getName() ) ) );
                Mage::getStoreConfig('system/store/name');
            */
            $this->addToLine( array('canal' => str_replace(' ', '',
                        Mage::app()->getStore( $order->getData('store_id') )->getFrontendName() ) ) ); // removing spaces help using this as an id



            $this->addToLine( array('date' => $this->dateFormat( $order->getCreatedAt() ) ) ); // $order->getUpdatedAt()
            $this->addToLine( array('client' => $order->getData('customer_id') ) );

            // main customer address
            $billing = $this->buildAddress( $order, 'shipping' );   // nh change 2016/12/01
            $this->addToLine( $billing );
            // we may possibly have a delivery address
/*
            // Shipping might nt exist or be the billing address
            if ( $shipping = $this->buildAddress( $order, 'shipping' )
              AND array_values( $billing ) != array_values( $shipping ) )
            {
                $this->addToLine( array('livPoint'=> json_encode( $shipping ) ) );
            }
*/
            // try to map local carrier specificity with an expedipro one
            $carrierItem = $order->getShippingMethod(); //->getData('shipping_description')
            $this->addToLine( array('livReseau' => ( isset( $carriers[ $carrierItem ]) ? $carriers[ $carrierItem ] : $carrierItem ) ) );

            if ( $point = $order->getData('soco_relay_point_code') ) {
                $this->addToLine( array(
                    'livPoint' => $point,
                    'livOption'=> $order->getData('soco_product_code')
                ));
            }


            // then manage order items
            try {
                //Mage_Sales_Model_Order_Item
                $orderItemCollection = $order->getAllItems();
                $panier = $this->buildPanier( $orderItemCollection );
            } catch (Exception $e) {
                // fallback : get order aggregates rather than detailed items
                $panier = array(
                        'pDesc'  => '',
                        'pQte'   => $order->getTotalQtyOrdered(),
                        'pVal'   => $order->getSubtotal(),  // amounts are without taxes
                        'pPoids' => $order->getWeight() );
            }
            $this->addToLine( $panier );

            $this->utils->answerAddData( $this->line['reference'], 'order', $this->line );
		}

        return true; // too verbose  $this->utils->logWrite( $this->utils->getAnswer() );
	}


    /**
     *
     *
     * @param Mage_Sales_Model_Order_Item collection $orderItemCollection

     *
     * @return array|bool
     */
    private function buildPanier( $orderItemCollection )
    {
        if ( ! $orderItemCollection )
            return false;

        $panier = array('pDesc'=>'', 'pQte'=>'','pVal'=>'','pPoids'=>'','pCateg'=>'');

        // whatever the method keep items aligns
        foreach ($orderItemCollection as $item)
        {
            // TODO : check this is the right way to discard size/color items
            if ( $item->getParentItemId() AND ! (float) $item->getBaseRowTotal() )
                continue;
            
//print_r( $item->getData() );
            // mb_detect_encoding('áéóú' /*ISO-8859-1*/ , 'UTF-8'); // 'UTF-8'
//            if ( ! mb_detect_encoding($itemArray['description'], 'UTF-8', true ) )
//                $itemArray['description'] = mb_convert_encoding($itemArray['description'], "UTF-8", mb_detect_encoding($itemArray['description']) );

            $itemOptions = ' ';
            $options = $item->getProductOptions();
            if ( $item->getProductType() == Mage_Catalog_Model_Product_Type::TYPE_CONFIGURABLE
                AND is_array($options)
                AND isset($options['attributes_info'])
                AND is_array($options['attributes_info'])
            ) {
                foreach ($options['attributes_info'] as $one)
                    $itemOptions .= $one['label'] .' '. $one['value'] .' ';
            }

            // add sku (reference/ean like) and name to the description
            $panier['pDesc'] .= $this->separatorInner
                                . ( ( $item->getProductType() == Mage_Catalog_Model_Product_Type::TYPE_CONFIGURABLE )
                                            ? $item->getProductOptionByCode('simple_sku')
                                            : $item->getSku() )
                                . ' '
                                . str_replace( $this->separatorInner, ' ', $item->getName() )
                                . $itemOptions
                                . str_replace( $this->separatorInner, ' ', $item->getProduct()->getDescription() );

            /* $item
                  ->getBasePrice() = unit price HT in catalog
                  ->getBasePriceInclTax() = unit price TTC
            */
            $panier['pVal']  .= $this->separatorInner . (float) $item->getBaseRowTotal(); // total HT (basePrice * qte
            $panier['pPoids'].= $this->separatorInner . (float) $item->getWeight();
            $panier['pQte']  .= $this->separatorInner . (int) $item->getQtyOrdered(); // ->getQtyShipped()
            $panier['pCateg'].= $this->separatorInner .
                ( ( $categ = $item->getProduct()->getCategory() ) ? $categ : '' ); // ->getParentCategory()

        }

        foreach ( $panier as $name => $string ) {
            $panier[ $name ] = ltrim($string, $this->separatorInner);
        }

        return $panier;
    }


    private function buildAddress( $order, $priority = 'billing' )
    {
        if ( $priority === 'shipping' )
            $method = array( 'getShippingAddress', 'getBillingAddress');
        else
            $method = array( 'getBillingAddress', 'getShippingAddress');


        if ( ! $addressModel = $order->{$method[0]}() )
            if ( ! $addressModel = $order->{$method[1]}() )
                return false;

        $genderCode = '';
        if ( $customer = $order->getCustomer() ) { //check if the customer exists or is not guest
            $genderCode = $customer->getGender();
            if ( isset( $this->gender[ $genderCode ]) )
                $genderCode = $this->gender[ $genderCode ];
/*
            $genderText = $customer->getResource()
                ->getAttribute('gender')
                ->getSource()
                ->getOptionText($customer->getData('gender')); //you get the label of the gender
*/
        }
        // note : lastname, firstname and email are order attributes, thus shared across addresses


        $isCompany = (bool) $addressModel->getCompany();

        return array(
            'civilite'		=> $genderCode,
            'prenom' 		=> $addressModel->getFirstname(),
            'nom' 			=> $addressModel->getLastname(),
            'entreprise' 	=> $isCompany,
            'entrepriseNom'	=> ( $isCompany ? $addressModel->getCompany() : ''),
//            'entrepriseTva'	=> '',
//            'optin'			=> true,
            'adresse1'		=> $addressModel->getStreet(1),
            'adresse2'		=> $addressModel->getStreet(2),
            'adresse3'		=> $addressModel->getStreet(3),
            'cp'			=> $addressModel->getPostcode(),
            'ville'			=> $addressModel->getCity(),
//            'etat'          => $addressModel->getRegionCode(),
            'pays'			=> $addressModel->getCountryId(),
            'instruction'	=> ( $addressModel->getStreet(4) ? $addressModel->getStreet(4) : $order->getData('customer_note') ),
            'telephoneF'	=> $addressModel->getTelephone(),
//            'telephoneM'	=> '',
            'email'			=> $order->getCustomerEmail()
        );
    }

/*
    private function addToFirstLine() {

        $firstLine = array();
        foreach( $this->template as $name => $useless )
            $firstLine[ $name ] = $name;

        //$line = mb_convert_encoding( $firstline, 'UTF-8' );
        $this->utils->answerAddData( null, 'order', $firstLine );
    }
*/


    private function addToLine( $value )
    {
/*
        if ( ! is_array( $value )
            AND isset( $this->template[ $name ] )
            AND ! $this->line[ $name ] )        // allows adding more info if current is meaningless
        {
            $this->line[$name] = $value;
            return;
        }
*/
        foreach ( $value as $key => $string ) {
            if ( isset($this->template[ $key ]) AND ! $this->line[ $key ] )
                $this->line[ $key ] = $string;
        }
    }

    private function resetLine( )
    {
        $this->utils->answerAddData( $this->line['reference'], 'order', $this->line );
        $this->line = $this->template;
    }

    private function dateFormat( $date ) {
        return date("Y-m-d", strtotime($date) );

    }
}
?>