<?php
/**
 * Class Expedipro_Synchro_Includes_Tellme
 *
 * This source file is subject to the MIT License
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/mit-license.php
 *
 * @category       Shipping
 * @package        Expedipro_Synchro
 * @author         Hilly
 * @copyright      Copyright (c) 2016
 * @license        http://opensource.org/licenses/mit-license.php MIT License
 */
class Expedipro_Synchro_Includes_Tellme
{
    private $utils;


    public function __construct( $utils = null ) {
        $this->utils = $utils;
    }

	/**
	 * Provide many configuration details to help remote debugging
     *
	 * @return boolean
	 */
    public function process( $action, $data )
    {
        switch ( $action ) {
            case 'tellMe' :
                return $this->tellMe( );
        }
        return false;
    }


    /**
     * Helps remote debugging
     *
     */
    protected function tellMe()
    {
        error_reporting(1); // useful ?

        try {
            $read  = Mage::getSingleton('core/resource')->getConnection('core_read');
            $version = $read->fetchOne('SELECT VERSION() AS "dbversion"');
        } catch (Exception $e) {
            $version = null;
        }
        try {
            if ( $module = mcrypt_module_self_test(MCRYPT_RIJNDAEL_128) )
                $moduleCBC = mcrypt_module_open ( MCRYPT_RIJNDAEL_128, "" , MCRYPT_MODE_CBC,""  );

        } catch (Exception $e) {
            $module = $moduleCBC =null;
        }


        $this->utils->answerAddData(
            null,
            'module',
            array(
                'name'          => 'magento',
                'version'       => $this->utils->getConfiguration('version'),
                'updateAllowed' => $this->utils->getConfiguration('updateAllowed'),

                'orderSeparator'=> $this->utils->getConfiguration('orderSeparator'),
                'orderStatus'   => $this->utils->getConfiguration('orderStatus'), // keep state for expedipro side ??
                'orderCarrier'  => $this->utils->getConfiguration('orderCarrier'),

                'deliveryTracking' => $this->utils->getConfiguration('deliveryTracking'),
                'deliveryStart' => $this->utils->getConfiguration('deliveryStart'),
                'deliveryDone'  => $this->utils->getConfiguration('deliveryDone'),

                'mcryptModule'  => ( $module ? 'ok' : 'error' ),
                'mcryptKey'     => print_r( mcrypt_enc_get_key_size( $moduleCBC ),true ),
                'mcryptIv'      => print_r( mcrypt_enc_get_iv_size( $moduleCBC ),true )
        ) );


        $pointInTime = new DateTime ( ); // prior to PHP 5.3, we can not chain with format

        $this->utils->answerAddData(
            null,
            'host',
            array(
                'name'      => 'magento',
                'version'   =>  Mage::getVersion(),
//                'version'   =>  Mage::getVersion(),
                'phpVersion'=>  phpversion(),
                'dbVersion' =>  $version,
                'time'      =>  $pointInTime->format('Y-m-d H:i:s') // UC : getOrder is always too old because the server is late or forward 
            )
        );

        $this->utils->answerAddData(
            null,
            'system',
            array(
                'phpInfo'       => $this->phpInfoArray( ),
//                'currentDir'    => $this->utils->getCurrentDir(),
                'currentScandir'=> scandir( $this->utils->getCurrentDir() ),
                'configScandir' => '',
                'included'      => get_included_files()
            )
        );
        return true;
    }


    /**
     * @param null $level for phpinfo
     * @return array
     */
    protected function phpInfoArray( $level = null )
    {
        if ( $level === null )
            $level = 4|8; //|32; // see http://php.net/manual/en/function.phpinfo.php
        ob_start();
        phpinfo( $level );
        $info_arr = array();
        $info_lines = explode("\n", strip_tags(ob_get_clean(), "<tr><td><h2>")); //executes both ob_get_contents() and ob_end_clean()

        $cat = "General";
        foreach($info_lines as $line)
        {
            // new cat?
            preg_match("~<h2>(.*)</h2>~", $line, $title) ? $cat = $title[1] : null;
            if( preg_match("~<tr><td[^>]+>([^<]*)</td><td[^>]+>([^<]*)</td></tr>~", $line, $val))
            {
                $info_arr[$cat][$val[1]] = trim($val[2]);
            }
            elseif(preg_match("~<tr><td[^>]+>([^<]*)</td><td[^>]+>([^<]*)</td><td[^>]+>([^<]*)</td></tr>~", $line, $val))
            {
                $info_arr[$cat][$val[1]] = array( "local" => trim($val[2]), "master" => trim($val[3]) );
            }
        }

        return $info_arr;
    }
}
?>