<?php

/**
 * Class Expedipro_Synchro_Includes_Utils
 *
 * This source file is subject to the MIT License
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/mit-license.php
 *
 * @category       Shipping
 * @package        Expedipro_Synchro
 * @author         Hilly
 * @copyright      Copyright (c) 2016
 * @license        http://opensource.org/licenses/mit-license.php MIT License
 */
class Expedipro_Synchro_Includes_Utils
{
    private $context;
    private $currentDir;
    private $answerError = array();
    private $answerData  = array();
    private $logFile;

    // class instances
    private $instances = array();


    public function __construct( $currentDir = null ) {
        
        $this->getCurrentDir( $currentDir );

        $this->logFile = $this->currentDir  .'/log/actions.log';
//        $this->logFileAlt = $this->currentDir  .'/log/actionsAlt.log';

//        include_once( $this->currentDir.'/includes/functions.php');
    }


    /**
     * Wrapper to eshop specificity
     *
     * @param null   $attribute
     * @param string $scope
     *
     * @return null
     */
    public function getConfiguration($attribute = null, $scope = 'expedipro' )
    {
        if ( ! $this->context ) {

            $this->context = new StdClass();
            /*
                        global $smarty, $cookie;
                        $context = new StdClass();
                        $context->smarty = $smarty;
                        $context->cookie = $cookie;
                        $context->language = new StdClass();
                        $context->language->id = ( isset($cookie->id_lang) ? $cookie->id_lang : Configuration::get('PS_LANG_DEFAULT'));
            //		$context->country = array('id_zone' => $defaultCountry->id_zone);
            */
            $this->context->expedipro = new StdClass();
            if ($config = Mage::getConfig()->getNode('default/expedipro_synchro')->asArray()) {
                $this->context->expedipro->token            = $config['source']['token'];
                $this->context->expedipro->updateAllowed    = $config['source']['update_allowed'];
                $this->context->expedipro->orderSeparator   = $config['order']['separator'];
                $this->context->expedipro->orderStatus      = explode(',' , $config['order']['status']); // related to  multiselect in system.xml
                $this->context->expedipro->orderCarrier     = explode(',' , $config['order']['carrier']);// related to  multiselect in system.xml
                $this->context->expedipro->deliveryTracking = $config['delivery']['tracking'];
                $this->context->expedipro->deliveryStart    = $config['delivery']['start'];
                $this->context->expedipro->deliveryDone     = $config['delivery']['done'];
            }
            // note the lcfirst vs ucfirst !
            if ($config = Mage::getConfig()->getNode('modules/Expedipro_Synchro')->asArray() )
                $this->context->expedipro->version = $config['version'];
        }

//    var_dump( $this->context->expedipro->deliveryStart );
        if ( isset( $this->context->$scope->$attribute ) )
            return $this->context->$scope->$attribute;

        return null;
        // otherwise any wrong typo would displayed token in tellMe
/*
        if ( isset( $this->context->$scope ) )
            return $this->context->$scope;

        return $this->context;
*/
    }


    // From prestashop classes/Tools.php
    public function getIsset($key)
    {
        if (!isset($key) OR empty($key) OR !is_string($key))
            return false;
        return isset($_POST[$key]) ? true : (isset($_GET[$key]) ? true : false);
    }


    public function getCurrentDir( $currentDir = null) {
        if ( $this->currentDir )
            return $this->currentDir;

        // php 5.3 compliant
        $this->currentDir = $currentDir ? $currentDir
                                        : Mage::getModuleDir('controllers', 'Expedipro_Synchro') .'/..';
        return $this->currentDir;
    }



    /**
     * Provide includes/* class instance
     *
     * @param string    $className  instance namee.g. 'credential', 'mapper' 
     * 
     * @return class instance
     */
    public function getInstance( $className ) {
        $className = ucfirst( strtolower( $className ) );

        return ( isset( $this->instances[$className])
                ? $this->instances[$className]
                : $this->loader($className) );
    }


    private function loader( $className ) {
        try {
            include ( $this->currentDir.'/includes/'. $className .'.php' ) ;
            $tmp = 'Expedipro_Synchro_Includes_'. $className;
            $this->instances[ $className ] = new $tmp( $this );
        } catch ( Exception $e)
        {
            $this->answerAddError( 'ACTUTILOA', 'Can not find file', $e->getMessage() );
            return false;
        }
        return  $this->instances[ $className ];
    }



    /**
     * JSON API like error
     *
     * @param int    $id
     * @param string $title     should not change from occurrence to occurrence
     * @param string $detail
     */
    public function answerAddError($id = '', $title = '', $detail = null )
    {
        $title  = ( empty( $title ) ? 'unknown' : $title); // required
        $id     = ( null === $id ? '' : $id); // null break json_decode() but could be "null" instead of ""
        if ( null === $detail )
            $err = array('id' => $id, 'title' => $title);
        else
            $err = array('id' => $id, 'title' => $title, 'detail' => $detail);

        $this->answerError[] = $err;
        // avoid writing to log when the error comes from here !
        if ( $id !== 'ACTUTIWRI' )
            $this->logWrite( var_export( $err, true ) );
    }

    /**
     * JSON API like data
     * IMPORTANT, spec define top level as a single resource type {} with related relationships into INCLUDED
     *      or a collection of one resource type [{},{}]
     * while here we could have an array of resources from different types
     *
     * @param int    $id
     * @param string $type     should not change from occurrence to occurrence
     * @param string $detail
     */
    public function answerAddData( $id = '', $type = '', $detail = null )
    {
        $type = ( empty( $type ) ? 'unknown' : $type); // required
        $id = ( null === $id ? '' : $id); // null break json_decode() but could be "null" instead of ""
        if ( null === $detail )
            $this->answerData[] = array('type' => $type, 'id' => $id);
        else
            $this->answerData[] = array('type' => $type, 'id' => $id, 'attributes' => $detail);
    }

    /**
     * Return a jsonapi like answer
     *
     * @param string $format
     * @return array|string
     */
    public function getAnswer( )
    {
        if ( ! empty( $this->answerError ) )
            return array('error' => $this->answerError );

        return array('data' => $this->answerData );
    }


    /**
     * write a message to the log
     *
     * @param string $message
     * @param bool $reset the log to 0
     *
     * @return bool
     */
    public function logWrite( $message = '', $reset = false ) {
 //       $pointInTime = new DateTime ( );
 //       $pointInTime->modify('-4 day'); // prior to PHP 5.3, we can not chain modify and format
        try {
//            if ( ! $this->logHandle ) // can not keep it open since not sure it would be closed afterward
            if ( ! $logHandle = fopen( $this->logFile, ( $reset ? 'w' : 'a') ) ) // b can be used as last mode option for binary writing
                throw new Exception('Can not open file '. $this->logFile .' with mode:: '. ( $reset ? 'w' : 'a') . ' in ' . __CLASS__);

            if ( is_array($message) )
                $message = var_export( $message, true );

            fwrite($logHandle, date('Y-m-d H:i:s ') . $message . PHP_EOL );
            fclose($logHandle);
            return true;
        } catch ( Exception $e ) {
            $this->answerAddError( 'ACTUTIWRI', 'log', $e->getMessage()); // update answerAddError with this id to avoid infinite loop
            return false;
        }
    }

    /**
     * read part of the log
     * beware getLog is defined in expediproMaintenance class
     *
     * @param bool $reset the log to 0
     *
     * @return bool|string
     */
    public function logRead( $truncate = false ) {

        try {
            $content = file_get_contents( $this->logFile );
//$this->logWrite( filesize( $this->logFile ) );

            if ( $truncate ) {
                if (!$logHandle = fopen($this->logFile, 'w'))
                    throw new Exception('Can not fopen file ' . $this->logFile . ' with mode w in ' . __CLASS__);
//                ftruncate($logHandle, 0);
                fclose($logHandle);
            }

        } catch ( Exception $e ) {
            $this->answerAddError( 'ACTUTIREA', 'log', $e->getMessage());
        }

        return $content; // better to get log anyaway

    }

    public function makeDir($path)
    {
        return is_dir($path) || mkdir($path);
    }


    public function copy_r( $path, $dest )
    {
        if( is_dir($path) )
        {
            @mkdir( $dest );    // @ since $dest might already exist
            $objects = scandir($path);
            if( sizeof($objects) > 0 )
            {
                foreach( $objects as $file )
                {
                    if( $file == "." || $file == ".." )
                        continue;
                    // go on
                    if( is_dir( $path.DIRECTORY_SEPARATOR.$file ) )
                        $this->copy_r( $path.DIRECTORY_SEPARATOR.$file, $dest.DIRECTORY_SEPARATOR.$file );
                    else
                        copy( $path.DIRECTORY_SEPARATOR.$file, $dest.DIRECTORY_SEPARATOR.$file );
                }
            }
            return true;
        }
        elseif( is_file($path) )
        {
            return copy($path, $dest);
        }

        return false;
    }


    public function delTree($dir) {
        if ( $dir === '/' OR empty($dir)  ) // avoid root level recursive delete !
            return false;

        $files = array_diff(scandir($dir), array('.','..'));
        foreach ($files as $file) {
            (is_dir("$dir/$file") && !is_link($dir)) ? $this->delTree("$dir/$file") : unlink("$dir/$file");
        }
        return rmdir($dir);
    }




//	https://github.com/ramsey/array_column/blob/master/src/array_column.php
// TODO : to be removed once in PHP 5.5

    /**
     * Returns the values from a single column of the input array, identified by
     * the $columnKey.
     *
     * Optionally, you may provide an $indexKey to index the values in the returned
     * array by the values from the $indexKey column in the input array.
     *
     * @param array $input		A multi-dimensional array (record set) from which to pull a column of values.
     * @param mixed $columnKey	The column of values to return. This value may be the integer key of the column you wish to retrieve, or it
     * 							may be the string key name for an associative array.
     * @param mixed $indexKey (Optional.) The column to use as the index/keys for the returned array. This value may be the integer key
     * 							of the column, or it may be the string key name.
     * @return array
     */
    public function array_column_54($input = null, $columnKey = null, $indexKey = null)
    {
        // Using func_get_args() in order to check for proper number of parameters and trigger errors exactly as the built-in array_column() does in PHP 5.5.
        $argc = func_num_args();
        $params = func_get_args();

        if ($argc < 2) {
            trigger_error("array_column() expects at least 2 parameters, {$argc} given", E_USER_WARNING);
            return null;
        }

        if (!is_array($params[0])) {
            trigger_error('array_column() expects parameter 1 to be array, ' . gettype($params[0]) . ' given', E_USER_WARNING);
            return null;
        }

        if (!is_int($params[1])
            && !is_float($params[1])
            && !is_string($params[1])
            && $params[1] !== null
            && !(is_object($params[1]) && method_exists($params[1], '__toString'))
        ) {
            trigger_error('array_column(): The column key should be either a string or an integer', E_USER_WARNING);
            return false;
        }

        if (isset($params[2])
            && !is_int($params[2])
            && !is_float($params[2])
            && !is_string($params[2])
            && !(is_object($params[2]) && method_exists($params[2], '__toString'))
        ) {
            trigger_error('array_column(): The index key should be either a string or an integer', E_USER_WARNING);
            return false;
        }

        $paramsInput = $params[0];
        $paramsColumnKey = ($params[1] !== null) ? (string) $params[1] : null;

        $paramsIndexKey = null;
        if (isset($params[2])) {
            if (is_float($params[2]) || is_int($params[2])) {
                $paramsIndexKey = (int) $params[2];
            } else {
                $paramsIndexKey = (string) $params[2];
            }
        }

        $resultArray = array();

        foreach ($paramsInput as $row) {

            $key = $value = null;
            $keySet = $valueSet = false;

            if ($paramsIndexKey !== null && array_key_exists($paramsIndexKey, $row)) {
                $keySet = true;
                $key = (string) $row[$paramsIndexKey];
            }

            if ($paramsColumnKey === null) {
                $valueSet = true;
                $value = $row;
            } elseif (is_array($row) && array_key_exists($paramsColumnKey, $row)) {
                $valueSet = true;
                $value = $row[$paramsColumnKey];
            }

            if ($valueSet) {
                if ($keySet) {
                    $resultArray[$key] = $value;
                } else {
                    $resultArray[] = $value;
                }
            }
        }
        return $resultArray;
    }



    /**
     * Read data from mapping/*.csv files
     *
     * @param string $semantic
     *
     * @return array
     */
    public function getMapping($semantic = null) {
        $mapping = array(
            'carrier'=> 'carrier.csv',
            'status' => 'status.csv' );

        if ( ! isset($mapping[ $semantic ]) )
            return array();

        if (  ! $file = fopen( $this->currentDir. "/mapping/". $mapping[ $semantic ], 'r') )
            return array();

        $structuredData = array();
        while ( ($rawData = fgetcsv ( $file, 4096, ',' )) !== false ) {
            if ( empty( $rawData ) or ! $rawData[0] or $rawData[0][0] ===';' )
                continue; // empty line is transformed in $rawData[0] = null

            $structuredData[ $rawData[0] ] = $rawData[1];
        }
        fclose( $file );

        return $structuredData;
    }
}
?>